{
This file is part of Unreal Tournament System Config version 210f.

Copyright (C) 2000, 2003 Simon McVittie.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

unit HugeExporter;

interface

procedure ExportBigDatToFiles(dat,path:string);

implementation

uses Classes,IniFiles,SysUtils,FileCtrl,Knowledge;

function ReadTerminatedString(Stream:TStream;Terminator:char):string;
var c:char;
Begin
Result:='';
try
  Stream.Read(c,1);
except
  on EReadError do c:=#0;
end;{try}
while c<>Terminator do
  begin
      Result:=Result+c;
    try
      Stream.Read(c,1);
    except
      on EReadError do c:=#0;
    end;{try}
  end;{while}
End;

function MakeLegalFilename(f:string):string;
const ControlChars = [#0..#31];
      ReservedChars = ['\','|','/','?','*','"',':','<','>'];
      IllegalChars = ControlChars + ReservedChars;
var i:integer;
Begin
      Result:=f;
      for i:=1 to length(Result) do
        if Result[i] in IllegalChars then Result[i]:='_';
      if Result='' then Result:='__NULL__';
End;

procedure ExportBigDatToFiles(dat,path:string);
var Stream:TFileStream; FileReader:TIniFile;
    i,j,NumFaces,NumObjects:integer;
    b:byte;
    s,t:string;
    more:boolean;
Begin
ForceDirectories(path+'\');
Stream:=TFileStream.Create(dat,fmOpenRead OR fmShareDenyWrite);
{read and discard header, we can assume this one's OK}
ReadTerminatedString(Stream,#0);
ReadTerminatedString(Stream,#0);

    {IN EACH MOD BEGIN:            }
{null terminated string: Name}
s:=ReadTerminatedString(Stream,#0);
more:= NOT (s='');
while More do begin
{nts:  Author}
t:=ReadTerminatedString(Stream,#0);
FileReader:=TIniFile.Create(path+'\'+MakeLegalFileName(s+' by '+t)+'.utsc.txt');
FileReader.WriteString(hMod,vModName,s);
FileReader.WriteString(hMod,vAuthor,t);
{nts:  Email}
s:=ReadTerminatedString(Stream,#0);
FileReader.WriteString(hMod,vEmail,'');
{nts:  WWW}
s:=ReadTerminatedString(Stream,#0);
FileReader.WriteString(hMod,vWWW,'');
{nts:  OtherContact}
s:=ReadTerminatedString(Stream,#0);
FileReader.WriteString(hMod,vOtherContact,'');
{nts:  Description}
s:=ReadTerminatedString(Stream,#0);
FileReader.WriteString(hMod,vModDesc,'');
{=====================}
{int:  number of SPs}
Stream.Read(numObjects,4);
FileReader.WriteInteger(hSPs,vNumSPs,0);
{?x nts: ServerPackage lines}
      for i:=1 to NumObjects do begin
        s:=ReadTerminatedString(Stream,#0);
        FileReader.WriteString(hSPs,vSPPrefix+IntToStr(i),'');
        end;
{nts  TypeHeader}
s:=ReadTerminatedString(Stream,#0);
FileReader.WriteString(hMod,vTypeHeader,s);
{nts DatafileCompilerName [use #13#0 for Psychic_313]}
s:=ReadTerminatedString(Stream,#0);
if (s=#13) then s:='Psychic_313';
FileReader.WriteString(hHeader,vDatafileCompilerName,s);

    {load up the models next}
    Stream.Read(numObjects,4);
    {int: number of models}
    FileReader.WriteInteger(hHeader,vNumModels,numObjects);
if (NumObjects>0) then
      for i:=1 to NumObjects do begin
      s:=hModelPrefix+IntToStr(i);
      {nts    BotClass:string;}
{nts    PlayerClass:string;}
{nts    SkinPackPrefix:string; }
{byte    (IsMale) + 128 if HasFaces true}
{nts    Desc:string;}
        t:=ReadTerminatedString(Stream,#0);
        FileReader.WriteString(s,vModelBotClass,t);
        {would write player class here}
        ReadTerminatedString(Stream,#0);
        {FileReader.WriteString(s,vModelPlayerClass,'');}
        t:=ReadTerminatedString(Stream,#0);
        FileReader.WriteString(s,vModelMesh,t);

        {b:=FileReader.ReadInteger(s,vModelIsMale,1) mod 128;
        if (FileReader.ReadInteger(s,vModelHasFaces,1)=1) then b:=b+128;
        Stream.Write(b,1); }
        Stream.Read(b,1);
        FileReader.WriteInteger(s,vModelIsMale,b mod 128);
        if ((b AND $80)=0) then FileReader.WriteInteger(s,vModelHasFaces,0)
        else FileReader.WriteInteger(s,vModelHasFaces,1);

        t:=ReadTerminatedString(Stream,#0);
        FileReader.ReadString(s,vModelDesc,t);
      end;{for i}


    {now try loading skins, these go in a cache for now}
    {
int: number of skins       }
Stream.Read(numObjects,4);
FileReader.WriteInteger(hHeader,vNumSkins,NumObjects);
    if (NumObjects>0) then
      for i:=1 to NumObjects do begin
        s:=hSkinPrefix+IntToStr(i);
        {nts    UTName:string;}
        t:=ReadTerminatedString(Stream,#0);
        FileReader.WriteString(s,vSkinName,t);
        {nts    Desc:string;   }
        t:=ReadTerminatedString(Stream,#0);
        FileReader.WriteString(s,vSkinDesc,t);
        {byte    IsTeamSkinnable:boolean; 0/255}
        Stream.Read(b,1);
        if b=0 then FileReader.WriteInteger(s,vSkinTeams,0)
        else FileReader.WriteInteger(s,vSkinTeams,1);
        {int number of faces                   }
        {nts Face1                               }
        {nts FaceDesc1                            }
        {nts Face2                                 }
        {nts FaceDesc2                              }
        Stream.Read(NumFaces,4);
        FileReader.WriteInteger(s,vSkinNumFaces,NumFaces);
        if (NumFaces>0) then
          for j:=1 to NumFaces do begin
            t:=ReadTerminatedString(Stream,#0);
            FileReader.WriteString(s,vFacePrefix+IntToStr(j),t);
            t:=ReadTerminatedString(Stream,#0);
            FileReader.WriteString(s,vFaceDescPrefix+IntToStr(j),t);
          end;{for j}
      end;{for i}

    {finally, do voices}
    Stream.Read(numObjects,4);
    FileReader.WriteInteger(hHeader,vNumVoices,numObjects);
    if (NumObjects>0) then
      for i:=1 to NumObjects do begin
        s:=hVoicePrefix+IntToStr(i);
        t:=readTerminatedString(Stream,#0);
        FileReader.WriteString(s,vVoiceDesc,t);
        {nts     Desc:string;}
        {nts    VoiceClass:string;}
        {byte    VoiceType}
        t:=ReadTerminatedString(Stream,#0);
        FileReader.ReadString(s,vVoiceClass,t);
        Stream.Read(b,1);
        FileReader.WriteInteger(s,vVoiceIsMale,b);
      end;{for i}
FileReader.Free;
s:=ReadTerminatedString(Stream,#0);
more:= NOT (s='');
end;{while}

Stream.Free;


End;

end.
