{
This file is part of Unreal Tournament System Config version 210f.

Copyright (C) 2000, 2003 Simon McVittie.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

UNIT SvPackFiles;

INTERFACE

uses classes, sysutils, dialogs;

{UT INI format:
---start
[section.name]
key=value
key=value

[section.name]
key=value
key=value
key=value
                       // note blank line
---eof}

procedure LoadServerPackages(Table:TStrings;const IniFile:string);

procedure GetPaths(Table:TStrings;IniFile:string);

{procedure AddFromPreset(Table:TStrings;PresetFile:string);}

procedure WriteServerPackages(Table:TStrings;const IniFile:string);

procedure VerifyThatPackagesExist(PckLst:TStrings;IniFile:string);

function PackageExists(Pack:String;IniFile:string):boolean;

function ExtractPackageName(UTResource:string):string;

{function ResourceIsInPaths(UTRes:string):boolean;}

IMPLEMENTATION
uses {YouDontHaveMod,} UTServerConfigGlobal;

const
  strServerPacksSection='[ENGINE.GAMEENGINE]';
  strwriteSPS='[Engine.GameEngine]';
  strwriteSP='ServerPackages=';
  strServerPacks='SERVERPACKAGES=';
  LengthSvPacks=15;

function FileIsInPaths(PathList:TStringList;upkg:string):boolean;
{- Nested function}
{|} function PathsReplacer(Path,StarReplacement:string):string;
{|} var i:integer;
{|} begin
{|} Result:='';
{|} for i:=1 to length(Path) do
{|}   if (Path[i]='*')
{|}     then Result:=Result+StarReplacement
{|}     else if (Path[i]='/')
{|}       then Result:=Result+'\'      {use DOS-style paths, not Unix-style}
{|}       else Result:=Result+Path[i];
{|} end;
{- end nesting}
var c:integer;
Begin
if (PathList.Count<1) then Result:=FALSE
else begin
  Result:=FALSE;
  for c:=0 to PathList.Count-1 do
    begin
      if Result then break else if FileExists(
            ExtractFileDir(Global.UTSysIni)+'\'+PathsReplacer(PathList[c],upkg))
        then Result:=TRUE;
    end;{for}
end;{else}
End;

procedure LoadServerPackages(Table:TStrings;const IniFile:string);
var Quit, QuitCurrent:boolean;
    F:text;
    S:string;
Begin
Quit:=FALSE;
{can't use normal INI techniques because UT INIs can have multiple values for
a key, as used for ServerPackages}
{ i) Read through until you reach [Engine.GameEngine] heading}
QuitCurrent:=FALSE;
assignFile(F,IniFile);
reset(F);
repeat
if eof(F) then Quit:=TRUE
else begin
  readln(F,S);
  if (uppercase(S)=strServerPacksSection) then QuitCurrent:=TRUE;
end;{if}
until Quit OR QuitCurrent;

QuitCurrent:=FALSE;
if not Quit then
{ ii) Get each line starting ServerPackages= and add the rest to the table}
repeat
if eof(F) then QuitCurrent:=TRUE
else begin
  readln(F,S);
  if (uppercase(copy(S,1,LengthSvPacks))=strServerPacks)
    then Table.Append(copy(S,(LengthSvPacks+1),length(S)))
  else if (S='') then QuitCurrent:=TRUE;
end;{else}
until QuitCurrent;
close(F);
End;

procedure WriteServerPackages(Table:TStrings;const IniFile:string);
var Buffer:TStringList;
    Done,Ready:boolean;
    F:text;
    S:string;
    i:integer;
Begin
Buffer:=TStringList.Create;
if (Table.Count>0) then begin
try

Done:=FALSE;
Ready:=FALSE;

AssignFile(F,IniFile);
Reset(F);

while not EOF(F) do
  begin
    readln(F,S);
    {ShowMessage('Read '+S);}
    if (UpperCase(S)=strServerPacksSection) then Ready:=TRUE
      else if Ready AND (S='')
             then begin
               {write serverpackages before it}
               {ShowMessage('Doing serverpackages');}
               for i:=0 to Table.Count-1 do
                 Buffer.Append(strWriteSP+Table.Strings[i]);
               Done:=TRUE;
               Ready:=FALSE;
             end;{else}
    if (UpperCase(Copy(S,1,LengthSvPacks))<>strServerPacks)
    then Buffer.Append(S);
    {ShowMessage('Wrote '+S);}
  end;{while}

CloseFile(F);

if not Done then ShowMessage('Error: This doesn''t seem to be a UT INI file')
else Buffer.SaveToFile(INIfile);

finally
Buffer.Free;
end;{try}
end;{if}
End;{proc}

(*procedure AddFromPreset(Table:TStrings;PresetFile:string);
var
    ModPkgList:TStringList;
    Paths:TStringList;
    Okay:boolean;
    i:integer;
    f:string;
const
    NullWebsite='<no website supplied, try http://www.fileplanet.com or '
                +'http://www.unreality.org>';
Begin
ModPkgList:=TStringList.Create;
Paths:=TStringList.Create;
try
if fileExists(PresetFile) then begin
  ModPkgList.LoadFromFile(PresetFile);
  if (ModPkgList.Count<4) then
    ShowMessage('Invalid preset file: '+PresetFile)
  else begin
    f:=Global.UTSysIni;
    if FileExists(f) then GetPaths(Paths,f);
    Okay:=TRUE;
    for i:=3 to ModPkgList.Count-1 do
      if not FileIsInPaths(Paths,ModPkgList[i]) then Okay:=FALSE;
    if Okay then begin
      for i:=3 to ModPkgList.Count-1 do
      Table.Append(ModPkgList[i]);
    end{if}
    else begin
      if (ModPkgList[2]='') then ModPkgList[2]:=NullWebsite;
      YouDontHaveModForm.PresetLabel.Caption:=ExtractFileName(PresetFile);
      YouDontHaveModForm.ModnameLabel.Caption:=ModPkgList[0];
      YouDontHaveModForm.UrlLabel.Caption:=ModPkgList[2];
      YouDontHaveModForm.ShowModal;
    end;{else}
  end;{else}
end;{if}
finally
ModPkgList.Free;
Paths.Free;
end;{try}
End;         *)

procedure GetPaths(Table:TStrings;IniFile:string);
var Quit, QuitCurrent:boolean;
    F:text;
    S:string;
const
    PathsSection='[CORE.SYSTEM]';
    PathsHdr='PATHS=';
    PathsHdrLen=6;
Begin
Quit:=FALSE;
{can't use normal INI techniques because UT INIs can have multiple values for
a key, as used for ServerPackages}
{ i) Read through until you reach [Core.System] heading}
QuitCurrent:=FALSE;
assignFile(F,IniFile);
reset(F);
repeat
if eof(F) then Quit:=TRUE
else begin
  readln(F,S);
  if (uppercase(S)=PathsSection) then QuitCurrent:=TRUE;
end;{if}
until Quit OR QuitCurrent;

QuitCurrent:=FALSE;
if not Quit then
{ ii) Get each line starting paths=,until next blank line,and add to the table}
repeat
if eof(F) then QuitCurrent:=TRUE
else begin
  readln(F,S);
  if (uppercase(copy(S,1,PathsHdrLen))=PathsHdr)
    then Table.Append(copy(S,(PathsHdrLen+1),length(S)))
  else if (S='') then QuitCurrent:=TRUE;
end;{else}
until QuitCurrent;
close(F);
End;

procedure VerifyThatPackagesExist(PckLst:TStrings;IniFile:string);
var PathTable:TStringList;
    i:integer;
Begin
PathTable:=TStringList.Create;
try
  GetPaths(PathTable,IniFile);
  if (PckLst.Count>0) then
    for i:=PckLst.Count-1 downto 0 do
      if not FileIsInPaths(PathTable,PckLst[i])
        then PckLst.Delete(i);
finally
PathTable.Free;
end;{try}
End;

function PackageExists(Pack:String;IniFile:string):boolean;
var PathTable:TStringList;
Begin
PathTable:=TStringList.Create;
try
GetPaths(PathTable,IniFile);
Result:=FileIsInPaths(PathTable,Pack);
finally
PathTable.Free;
end;{try}
End;

function ExtractPackageName(UTResource:string):string;
Begin
 if (Pos('.',UTResource)=0) then Result:=''
 else Result:=Copy(UTResource,1,Pos('.',UTResource)-1);
End;

END.
