{
This file is part of Unreal Tournament System Config version 210f.

Copyright (C) 2000, 2003 Simon McVittie.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

UNIT TBigModFileDefine;
INTERFACE

uses T313ObjectDefine,TBotModelDefine,TBotSkinDefine,TBotVoiceDefine,
     TModDetailsDefine,Classes,SysUtils;

const UTSCBigDataHeaderString = 'pseudorandom.co.uk_UTSC_Binary_v210';

type TBigModFile = class(TObject)
     public
       IExist:boolean;
       Models:TBotModelList;
       Skins:TBotSkinList;
       Voices:TBotVoiceList;
       ModDetails:TModDetails;
       constructor CreateAndRead(Stream:TStream);
       constructor Create;
       destructor Destroy; override;
     end;

IMPLEMENTATION


{FILE FORMAT =========================================

plain ASCII: 'pseudorandom.co.uk_UTSC_Binary_v210'
#0
ignore this section (future use)
#0           }

{byte: no of mods}

{IN EACH MOD BEGIN:            }
{null terminated string: Name}
{nts:  Author}
{nts:  Email}
{nts:  WWW}
{nts:  OtherContact}
{nts:  Description}
{byte:  number of SPs}
{?x nts: ServerPackage lines}
{    [installed:bool not appropriate]}
{nts  TypeHeader}
{  [UtscDatafile:string;] n/a}
{nts DatafileCompilerName [use #13#0 for Psychic_313]}

{byte: number of models}

{?x:                          }
{nts    BotClass:string;}
{nts    PlayerClass:string;}
{nts    SkinPackPrefix:string; }
{byte    (IsMale) + 128 if HasFaces true}
{nts    Desc:string;}
{       [not in mod-details bots    Skins:TBotSkinList;]}
{
byte: number of skins       }

{?x:}
{nts    UTName:string;}
{nts    Desc:string;   }
{byte    IsTeamSkinnable:boolean; 0/255}
{byte number of faces                   }
{nts Face1                               }
{nts FaceDesc1                            }
{nts Face2                                 }
{nts FaceDesc2                              }

{byte: number of voices  }

{?x: }
{nts     Desc:string;}
{nts    VoiceClass:string;}
{byte    VoiceType}

{END
next mod, or #0 for eof

}

function ReadTerminatedString(Stream:TStream;Terminator:char):string;
var c:char;
Begin
Result:='';
try
  Stream.Read(c,1);
except
  on EReadError do c:=#0;
end;{try}
while c<>Terminator do
  begin
      Result:=Result+c;
    try
      Stream.Read(c,1);
    except
      on EReadError do c:=#0;
    end;{try}
  end;{while}
End;

constructor TBigModFile.Create;
Begin
Models:=TBotModelList.Create;
Skins:=TBotSkinList.Create;
Voices:=TBotVoiceList.Create;
ModDetails:=TModDetails.Create;
End;

constructor TBigModFile.CreateAndRead(Stream:TStream);
var i,j,Count,Count2:longint;
    b:byte;
    s:string;
    Model:TBotModel;
    Skin:TBotSkin;
    Voice:TBotVoice;
Begin
Self.Create;
try
{we're already inside the stream at this point, some other object has done
the header processing}

{null terminated string: Name (will only be null for EOF)}

s:=ReadTerminatedString(Stream,#0);
if s='' then IExist:=FALSE
else begin
IExist:=TRUE;
ModDetails.Name:=s;
{nts:  Author}
ModDetails.Author:=ReadTerminatedString(Stream,#0);
{nts:  Email       }
ModDetails.Email:=ReadTerminatedString(Stream,#0);
{nts:  WWW         }
ModDetails.WWW:=ReadTerminatedString(Stream,#0);
{nts:  OtherContact}
ModDetails.OtherContact:=ReadTerminatedString(Stream,#0);
{nts:  Description }
ModDetails.Description:=ReadTerminatedString(Stream,#0);
{int:  number of SPs}
Stream.Read(Count,4);
{?x nts: ServerPackage lines}
for i:=1 to Count do begin
  s:=ReadTerminatedString(Stream,#0);
  if s<>'' then ModDetails.ServerPackages.Append(s);
   end;{for}
{    [installed:bool not appropriate]}
ModDetails.IsInstalled:=FALSE; {for now}
ModDetails.TypeHeader:=ReadTerminatedString(Stream,#0);
{nts  TypeHeader}
{  [UtscDatafile:string;] n/a}
ModDetails.UtscDatafile:='';
{nts DatafileCompilerName [use #13#0 for Psychic_313]}
s:=ReadTerminatedString(Stream,#0);
if s=#13 then s:='Psychic_313';
ModDetails.DatafileCompilerName:=s;

{MODELS}

Stream.Read(Count,4);
for i:=1 to Count do begin
Model:=TBotModel.Create;
{nts    BotClass:string;}
Model.BotClass:=ReadTerminatedString(Stream,#0);
{nts    PlayerClass:string;}
Model.PlayerClass:=ReadTerminatedString(Stream,#0);
{nts    SkinPackPrefix:string; }
Model.SkinPackPrefix:=ReadTerminatedString(Stream,#0);
{byte    (IsMale) + 128 if HasFaces true}
Stream.Read(b,1);
Model.IsMale:= b AND $7f;
Model.HasFaces:= ((b AND $80)<>0);
{nts    Desc:string;}
Model.Desc:=ReadTerminatedString(Stream,#0);
{       [not in mod-details bots    Skins:TBotSkinList;]}
Models.Append(Model);
end;{for all bots}

{SKINS}
Stream.Read(Count,4);
for i:=1 to Count do begin
Skin:=TBotSkin.Create;
{nts    UTName:string;}
Skin.UTName:=ReadTerminatedString(Stream,#0);
{nts    Desc:string;}
Skin.Desc:=ReadTerminatedString(Stream,#0);
{byte    IsTeamSkinnable:boolean; 0/255}
Stream.Read(b,1);
Skin.IsTeamSkinnable:=(b<>0);
{int number of faces}
Stream.Read(Count2,4);
for j:=1 to Count2 do begin
{nts Face}
Skin.Faces.Append(ReadTerminatedString(Stream,#0));
{nts FaceDesc}
Skin.FaceDescs.Append(ReadTerminatedString(Stream,#0));
end;{for all faces}
Skins.Append(Skin);
end;{for all skins}

{VOICES}
Stream.Read(Count,4);
for i:=1 to Count do begin
Voice:=TBotVoice.Create;
{nts     Desc:string;}  Voice.Desc:=ReadTerminatedString(Stream,#0);
{nts    VoiceClass:string;}Voice.VoiceClass:=ReadTerminatedString(Stream,#0);
{byte    VoiceType}     Stream.Read(j,1); Voice.VoiceType:=j;
Voices.Append(Voice);
end;{for all voices}

end;{if}
except
  on EReadError do begin
    Models.Clear;
    Skins.Clear;
    Voices.Clear;
    ModDetails.Free;
    ModDetails:=TModDetails.Create;
    end;{on eReadError}
end;

End;

destructor TBigModFile.Destroy;
Begin
Models.Free;
Skins.Free;
Voices.Free;
ModDetails.Free;
End;

END.
