{
This file is part of Unreal Tournament System Config version 210f.

Copyright (C) 2000, 2003 Simon McVittie.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

UNIT UTServerConfigGlobal;

INTERFACE

type
  TUTSCGlobals=class(TObject)
    public
    function MyFolder:string;
    function UTBase:string;
    function UTSysIni:string;
    function UTUserIni:string;
    function UTExecutable:string;
  end;{class}

procedure UTServerConfigStartup;
procedure UTServerConfigOptions;
procedure UTServerConfigShutdown;

var global_MyFolder:string;
    global_UTBaseFolder:string;
    global_UTSysIni:string;
    global_UTUserIni:string;
    global_UTExecutable:string;
    global_UTExePrompt:boolean;
    global_UTIniPrompt:boolean;
    global_UTUserPrompt:boolean;
    global_Initialized:boolean;
    global_autoexit:boolean;
    global_botfolder:string;
    Global:TUTSCGlobals;

IMPLEMENTATION
uses Registry,Windows,SysUtils,Forms,FirstRunOptions,Options,Controls,
     Dialogs,MainMenu,FileCtrl,UTSharedUtils1,Knowledge,SvPack;
const MyRootKey=HKEY_CURRENT_USER;
      MyTopLevelKey='\Software';
      MyKeyPseudorandom='pseudorandom.co.uk';
      MyKeyUpToPseudorandom=MyTopLevelKey+'\'+MyKeyPseudorandom;
      MyKeyParadox='Paradox Productions';
      MyKeyUpToParadox=MyKeyUpToPseudorandom+'\'+MyKeyParadox;
      MyKeyApp='UTSC';
      MyKeyUpToApp=MyKeyUpToParadox+'\'+MyKeyApp;
      MyKeyVersion='version210';
      MyKey=MyKeyUpToApp+'\'+MyKeyVersion;
      bExe='PromptToFindExe';
      bIni='PromptToFindIni';
      bUser='PromptToFindUser';
      vExe='ExePath';
      vIni='IniPath';
      vUser='UserPath';
      bAutoExit='ExitOnRunUT';
      vBase='UtBasePath';

{class}
function TUTSCGlobals.MyFolder;
Begin
  Result:=global_MyFolder;
End;

function TUTSCGlobals.UTBase;
Begin
  Result:=global_UTBaseFolder;
End;

function TUTSCGlobals.UTSysIni;
Begin
  if global_UtIniPrompt then
    begin
      MainMenuForm.GlobalOpenDlg.Title:='Select UT system INI';
      MainMenuForm.GlobalOpenDlg.FileName:=global_UTSysIni;
      MainMenuForm.GlobalOpenDlg.Filter:='UT system INI (*.ini)|*.ini';
      MainMenuForm.GlobalOpenDlg.DefaultExt:='ini';
      MainMenuForm.GlobalOpenDlg.InitialDir:=ExtractFileDir(global_UtSysIni);
      if not MainMenuForm.GlobalOpenDlg.Execute then
        Result:='<cancelled>'
      else Result:=MainMenuForm.GlobalOpenDlg.FileName;
    end
  else Result:=global_UtSysIni;
End;

function TUTSCGlobals.UTUserIni;
Begin
  if global_UtUserPrompt then
    begin
      MainMenuForm.GlobalOpenDlg.Title:='Select UT user INI';
      MainMenuForm.GlobalOpenDlg.FileName:=global_UTUserIni;
      MainMenuForm.GlobalOpenDlg.Filter:='UT user INI (*.ini)|*.ini';
      MainMenuForm.GlobalOpenDlg.DefaultExt:='ini';
      MainMenuForm.GlobalOpenDlg.InitialDir:=ExtractFileDir(global_UtUserIni);
      if not MainMenuForm.GlobalOpenDlg.Execute then
        Result:='<cancelled>'
      else Result:=MainMenuForm.GlobalOpenDlg.FileName;
    end
  else Result:=global_UtUserIni;
End;

function TUTSCGlobals.UTExecutable;
Begin
  if global_UtExePrompt then
    begin
      MainMenuForm.GlobalOpenDlg.Title:='Select UT executable';
      MainMenuForm.GlobalOpenDlg.FileName:=global_UTExecutable;
      MainMenuForm.GlobalOpenDlg.Filter:='UT executable (*.exe)|*.exe';
      MainMenuForm.GlobalOpenDlg.DefaultExt:='exe';
      MainMenuForm.GlobalOpenDlg.InitialDir:=ExtractFileDir(global_UtExecutable);
      if not MainMenuForm.GlobalOpenDlg.Execute then
        Result:='<cancelled>'
      else Result:=MainMenuForm.GlobalOpenDlg.FileName;
    end
  else Result:=global_UtExecutable;
End;


{non-interface}
function GetUTBaseFolder:string;
var Reg:TRegistry;
const UtKey='\Software\Unreal Technology\Installed Apps\UnrealTournament';
      UtVal='Folder';
      UtRoot=HKEY_LOCAL_MACHINE;
      defaultUT='c:\UnrealTournament';
Begin
  Reg:=TRegistry.Create;
  try
  Reg.RootKey:=UTRoot;
  Result:='';
  if Reg.OpenKey(UtKey,FALSE) then
    if Reg.ValueExists(UtVal) then Result:=Reg.ReadString(UtVal);
  finally
  Reg.Free;
  end;{try}
  if (Result='') then Result:=defaultUT;
End;

procedure LoadGlobalBooleans;
var Reg:TRegistry;

Begin
  Reg:=TRegistry.Create;
  try
  Reg.RootKey:=MyRootkey;
  if Reg.OpenKey(MyKey,False) then begin
    if Reg.ValueExists(bExe) then global_UtExePrompt:=Reg.ReadBool(bExe);
    if Reg.ValueExists(bIni) then global_UtIniPrompt:=Reg.ReadBool(bIni);
    if Reg.ValueExists(bUser) then global_UtUserPrompt:=Reg.ReadBool(bUser);
    if Reg.ValueExists(bAutoExit) then global_Autoexit:=Reg.ReadBool(bAutoExit);
    end;{if}
  finally
  Reg.Free;
  end;{try}
End;

procedure SaveGlobalBooleans;
var Reg:TRegistry;
Begin
  Reg:=TRegistry.Create;
  try
  Reg.RootKey:=MyRootKey;
  Reg.OpenKey(MyKey,True);
  Reg.WriteBool(bExe,global_UtExePrompt);
  Reg.WriteBool(bIni,global_UtIniPrompt);
  Reg.WriteBool(bUser,global_UtUserPrompt);
  Reg.WriteBool(bAutoExit,global_AutoExit);
  finally
  Reg.Free;
  end;{try}
End;

procedure LoadGlobalStrings;
var Reg:TRegistry;
const MydocsKey='\Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders';
      MyDocsRoot=HKEY_CURRENT_USER;
      vMydocs='Personal';
Begin
  Reg:=TRegistry.Create;
  try
  Reg.RootKey:=MyRootKey;
  if Reg.OpenKey(MyKey,False) then begin
    if Reg.ValueExists(vExe) then global_UtExecutable:=Reg.ReadString(vExe);
    if Reg.ValueExists(vIni) then global_UtSysIni:=Reg.ReadString(vIni);
    if Reg.ValueExists(vUser) then global_UtUserIni:=Reg.ReadString(vUser);
    if Reg.ValueExists(vBase) then global_UtBaseFolder:=Reg.ReadString(vBase);
    end;{if}
  Reg.RootKey:=MyDocsRoot;
  if Reg.OpenKey(MydocsKey,FALSE) then
    if Reg.ValueExists(vMydocs) then
      global_botfolder:=RemoveQuotes(Reg.ReadString(vMyDocs));  {My Documents, for Win'98 users}
  finally
  Reg.Free;
  end;{try}
End;

procedure SaveGlobalStrings;
var Reg:TRegistry;
Begin
  Reg:=TRegistry.Create;
  try
  Reg.RootKey:=MyRootKey;
  Reg.OpenKey(MyKey,True);
  Reg.WriteString(vExe,global_UtExecutable);
  Reg.WriteString(vIni,global_UtSysIni);
  Reg.WriteString(vUser,global_UtUserIni);
  Reg.WriteString(vBase,global_UtBaseFolder);
  finally
  Reg.Free;
  end;{try}
End;

{interface}

procedure UTServerConfigStartup;
  var RegInterface:TRegistry;
Begin
  RegInterface:=TRegistry.Create;
  try
  RegInterface.RootKey:=MyRootKey;
  global_Initialized:=RegInterface.KeyExists(MyKey);
  finally
  RegInterface.Free;
  end;{try}
  if not global_Initialized then
    begin
      {first run}
      Application.CreateForm(TFirstRunOptionsForm,FirstRunOptionsForm);
      try
      global_Initialized:=TRUE;
      global_UTBaseFolder:=GetUTBaseFolder;
      global_UTSysIni:=global_UTBaseFolder+'\System\UnrealTournament.ini';
      global_UTUserIni:=global_UTBaseFolder+'\System\User.ini';
      global_UTExecutable:=global_UTBaseFolder+'\System\UnrealTournament.exe';
      SaveGlobalStrings;
      FirstRunOptionsForm.Edit1.Text:=global_UTBaseFolder;
      FirstRunOptionsForm.ShowModal;
      global_UTExePrompt:=FALSE;
      global_UtIniPrompt:=FALSE;
      global_UtUserPrompt:=FALSE;
      global_AutoExit:=TRUE;
      SaveGlobalBooleans;
      finally
        FirstRunOptionsForm.Release;
      end;{try}
    end{if}
  else {load defaults in case the registry access, below, fails}
    begin
      global_UTExePrompt:=FALSE;
      global_UtIniPrompt:=FALSE;
      global_UtUserPrompt:=FALSE;
      global_autoexit:=TRUE;
      global_UTBaseFolder:='c:\unrealtournament';
      global_UTSysIni:=global_UTBaseFolder+'\System\UnrealTournament.ini';
      global_UTUserIni:=global_UTBaseFolder+'\System\User.ini';
      global_UTExecutable:=global_UTBaseFolder+'\System\UnrealTournament.exe';
    end;{else}
  global_MyFolder:=ExtractFileDir(Application.ExeName);
  global_BotFolder:='c:\';
  LoadGlobalStrings;
  LoadGlobalBooleans;
  Knowledge.LoadMasterList;
  {debug ShowMessage(Application.MainForm.Caption);}
  MainMenuForm.Show;
End;

procedure UTServerConfigOptions;
Begin
  OptionsForm.UtBaseEdit.Text:=global_UtBaseFolder;
  OptionsForm.UTuseredit.Text:=global_UtUserIni;
  OptionsForm.UtUserPromptCheckbox.Checked:=global_UtUserPrompt;
  OptionsForm.UTIniEdit.Text:=global_UtSysIni;
  OptionsForm.UtIniPromptCheckbox.Checked:=global_UtIniPrompt;
  OptionsForm.UtExeEdit.Text:=global_UtExecutable;
  OptionsForm.UtExePromptCheckbox.Checked:=global_UtExePrompt;
  OptionsForm.AutoExitCheckbox.Checked:=global_Autoexit;
  if (OptionsForm.ShowModal=mrOK) then begin
      global_UtBaseFolder:=OptionsForm.UtBaseEdit.Text;
      global_UtUserIni:=OptionsForm.UTuseredit.Text;
      global_UtUserPrompt:=OptionsForm.UtUserPromptCheckbox.Checked;
      global_UtSysIni:=OptionsForm.UTIniEdit.Text;
      global_UtIniPrompt:=OptionsForm.UtIniPromptCheckbox.Checked;
      global_UtExecutable:=OptionsForm.UtExeEdit.Text;
      global_UtExePrompt:=OptionsForm.UtExePromptCheckbox.Checked;
      global_Autoexit:=OptionsForm.AutoExitCheckbox.Checked;
  end;{if}
End;

procedure UTServerConfigShutdown;
Begin
  MainMenuForm.ShuttingDown:=TRUE;
  SaveGlobalStrings;
  SaveGlobalBooleans;
  Application.Terminate;
End;

END.
