{
This file is part of Unreal Tournament System Config version 210f.

Copyright (C) 2000, 2003 Simon McVittie.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

unit MainMenu;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, UTServerConfigGlobal,Knowledge, ComCtrls,
  Menus, ToolWin;

type
  TMainMenuForm = class(TForm)
    GlobalOpenDlg: TOpenDialog;
    GlobalSaveDlg: TSaveDialog;
    StatusBar1: TStatusBar;
    UtscMainMenu: TMainMenu;
    ServerPackages1: TMenuItem;
    UTSCMenu: TMenuItem;
    Exit1: TMenuItem;
    RunUT1: TMenuItem;
    HelpMenu: TMenuItem;
    ContextHelpOption: TMenuItem;
    N5: TMenuItem;
    HelpContentsOption: TMenuItem;
    HelpAboutOption: TMenuItem;
    SaveAsOption: TMenuItem;
    SaveOption: TMenuItem;
    OpenBotlibOption: TMenuItem;
    NewBotlibOption: TMenuItem;
    Optionsuninstall1: TMenuItem;
    N7: TMenuItem;
    N8: TMenuItem;
    WindowMenu: TMenuItem;
    CloseCurrentOption: TMenuItem;
    LibrarySaveDialog: TSaveDialog;
    LibraryOpenDialog: TOpenDialog;
    Edit1: TMenuItem;
    NewBotLibBlankOption: TMenuItem;
    NewBotLibFromUTIniOption: TMenuItem;
    WindowCascadeOption: TMenuItem;
    WindowTileOption: TMenuItem;
    WindowArrangeiconsOption: TMenuItem;
    WeaponsFixerOption: TMenuItem;
    DatafileNewOption: TMenuItem;
    DatafileOpenOption: TMenuItem;
    Panel1: TPanel;
    ToolBar1: TToolBar;
    SpeedButton6: TSpeedButton;
    SpeedButton7: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton1: TSpeedButton;
    SpeedButton3: TSpeedButton;
    TitlePanel: TPanel;
    Label1: TLabel;
    Image1: TImage;
    NewSubmenu: TMenuItem;
    OpenSubmenu: TMenuItem;
    UtilsSubmenu: TMenuItem;
    ServerHeader: TMenuItem;
    ClientHeader: TMenuItem;
    N1: TMenuItem;
    SpeedButton4: TSpeedButton;
    SpeedButton5: TSpeedButton;
    SaveBotsBtn: TSpeedButton;
    Image2: TImage;
    Image3: TImage;
    procedure ConfigSPsBtnClick(Sender: TObject);
    procedure ExitBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure OptionsBtnClick(Sender: TObject);
    procedure RunUTBtnClick(Sender: TObject);
    procedure AboutBtnClick(Sender: TObject);
    {}
    procedure SpawnBotLibForm(title:string);
    procedure NewBotlibOptionClick(Sender: TObject);
    procedure OpenBotlibOptionClick(Sender: TObject);
    procedure LibraryOpenDialogClose(Sender: TObject);
    procedure LibrarySaveDialogClose(Sender: TObject);
    procedure HelpContentsOptionClick(Sender: TObject);
    procedure HelpAboutOptionClick(Sender: TObject);
    procedure WeaponsFixerOptionClick(Sender: TObject);
    procedure DatafileNewOptionClick(Sender: TObject);
    procedure InformChangedKnowledge;
    procedure WindowCascadeOptionClick(Sender: TObject);
    procedure WindowTileOptionClick(Sender: TObject);
    procedure WindowArrangeiconsOptionClick(Sender: TObject);
    procedure NewBotLibFromUTIniOptionClick(Sender: TObject);
    procedure SaveOptionClick(Sender: TObject);
    procedure ChangeFormFocus(Sender: TObject);
    procedure CloseCurrentOptionClick(Sender: TObject);
    procedure SaveAsOptionClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    SvPackOpen:boolean;
    WeaponsFixerOpen:boolean;
    ShuttingDown:boolean;
  end;

var
  MainMenuForm: TMainMenuForm;

implementation
uses MdiChildBotLib,SvPack,Options,UtSharedUtils1,About,DatafileMaker
,WeaponsFixer{,TPresetDialogDefine};
{$R *.DFM}

procedure TMainMenuForm.ConfigSPsBtnClick(Sender: TObject);
begin
  if not SvPackOpen then begin
  {ShowMessage('DEBUG: SvPack is NIL, spawning a new one');}
  Application.CreateForm(TSvPackForm,SvPackForm);
  SvPackOpen:=TRUE;
  end{if}
  else SvPackForm.WindowState:=wsNormal;
end;

procedure TMainMenuForm.ExitBtnClick(Sender: TObject);
begin
  close;
end;

procedure TMainMenuForm.FormShow(Sender: TObject);
begin
Screen.OnActiveFormChange:=Self.ChangeFormFocus;
end;

procedure TMainMenuForm.FormCreate(Sender: TObject);
begin
SvPackOpen:=FALSE;
WeaponsFixerOpen:=FALSE;
ShuttingDown:=FALSE;
end;

procedure TMainMenuForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
ShuttingDown:=TRUE;
UTServerConfigShutdown;
end;

procedure TMainMenuForm.OptionsBtnClick(Sender: TObject);
begin
UTServerConfigOptions;
end;

procedure TMainMenuForm.RunUTBtnClick(Sender: TObject);
var f:string;
begin
f:=Global.UTExecutable;
  if FileExists(f)
    then begin
      ExecuteFile(f,'',ExtractFileDir(f),SW_SHOW);
      if global_autoexit then Self.Close;
    end;{if}
end;

procedure TMainMenuForm.AboutBtnClick(Sender: TObject);
begin
Aboutbox.ShowModal;
end;

procedure TMainMenuForm.SpawnBotLibForm(title:string);
var
  Child: TChildBotLibForm;
const
  Suffix = ' - Bot Library Manager';
Begin
  { create a new MDI child window }
  Child := TChildBotLibForm.Create(Application);
  if title <> '' then Child.OpenFile(title);
End;

procedure TMainMenuForm.NewBotlibOptionClick(Sender: TObject);
begin
  SpawnBotlibForm('');
end;

procedure TMainMenuForm.OpenBotlibOptionClick(Sender: TObject);
begin
  if LibraryOpenDialog.Execute then
  begin
    SpawnBotlibForm(LibraryOpenDialog.Filename);
  end;
end;

procedure TMainMenuForm.LibraryOpenDialogClose(Sender: TObject);
begin
if FileExists(LibraryOpenDialog.Filename) then global_botfolder:=ExtractFileDir(LibraryOpenDialog.Filename);
end;

procedure TMainMenuForm.LibrarySaveDialogClose(Sender: TObject);
begin
if FileExists(LibrarySaveDialog.Filename) then global_botfolder:=ExtractFileDir(LibrarySaveDialog.Filename);
end;

procedure TMainMenuForm.HelpContentsOptionClick(Sender: TObject);
begin
AboutBox.GetHelp(helpContents);
end;

procedure TMainMenuForm.HelpAboutOptionClick(Sender: TObject);
begin
AboutBox.GetHelp(helpAbout);
end;

procedure TMainMenuForm.WeaponsFixerOptionClick(Sender: TObject);
begin
  if not WeaponsFixerOpen then begin
  Application.CreateForm(TWeaponsPriorityFixer,WeaponsPriorityFixer);
  WeaponsFixerOpen:=TRUE;
  WeaponsPriorityFixer.FixPriorities(global.UTUserIni);
  end{if}
  else WeaponsPriorityFixer.WindowState:=wsNormal;
end;

procedure TMainMenuForm.DatafileNewOptionClick(Sender: TObject);
var Child:TDatafileWizard;
begin
  Application.CreateForm(TDatafileWizard,Child);
  Child.CreateNew;
  Child.Show;
  Knowledge.LoadMasterList;
  InformChangedKnowledge;
end;

procedure TMainMenuForm.InformChangedKnowledge;
{tell forms the Knowledge has changed}
var i:integer;
Begin
Knowledge.LoadMasterList;
  for i:=Self.MDIChildCount-1 downto 0 do begin
    if Self.MDIChildren[i] IS TChildBotLibForm then
      TChildBotLibForm(Self.MDIChildren[i]).ReloadKnowledge(Self);
    if Self.MDIChildren[i] IS TSvPackForm then
      TSvPackForm(Self.MDIChildren[i]).UpdateChecklist(Self);
  end;{for}
End;

procedure TMainMenuForm.WindowCascadeOptionClick(Sender: TObject);
begin
Cascade;
end;

procedure TMainMenuForm.WindowTileOptionClick(Sender: TObject);
begin
Tile;
end;

procedure TMainMenuForm.WindowArrangeiconsOptionClick(Sender: TObject);
begin
ArrangeIcons;
end;

procedure TMainMenuForm.NewBotLibFromUTIniOptionClick(Sender: TObject);
var
  Child: TChildBotLibForm;
Begin
  { create a new MDI child window }
  Application.CreateForm(TChildBotLibForm,Child);
  Child.ImportINIBtnClick(Sender);
End;

procedure TMainMenuForm.SaveOptionClick(Sender: TObject);
begin
if Self.ActiveMDIChild is TChildBotlibForm then
   TChildBotlibForm(Self.ActiveMDIChild).SaveBtnClick(Sender)
else if Self.ActiveMDIChild is TSvPackForm then
  TSvPackForm(ActiveMDIChild).SaveBtnClick(Sender)
else if Self.ActiveMDIChild is TWeaponsPriorityFixer then
  TWeaponsPriorityFixer(ActiveMDIChild).BitBtn7Click(Sender);
end;

procedure TMainMenuForm.ChangeFormFocus(Sender: TObject);
Begin
if not ShuttingDown then begin
SaveBotsBtn.Enabled:=FALSE;
if Self.ActiveMDIChild=NIL then
   begin
     Self.SaveOption.Enabled:=FALSE;
     Self.SaveAsOption.Enabled:=FALSE;
     Self.CloseCurrentOption.Enabled:=FALSE;
   end
else
if Self.ActiveMDIChild is TChildBotlibForm then
   begin
     Self.SaveOption.Enabled:=TRUE;
     Self.SaveAsOption.Enabled:=TRUE;
     Self.CloseCurrentOption.Enabled:=TRUE;
     SaveBotsBtn.Enabled:=TRUE;
   end
else
if Self.ActiveMDIChild is TDatafileWizard then
   begin
     Self.SaveOption.Enabled:=FALSE;
     Self.SaveAsOption.Enabled:=FALSE;
     Self.CloseCurrentOption.Enabled:=TRUE;
   end
else
if Self.ActiveMDIChild is TSvPackForm then
   begin
     Self.SaveOption.Enabled:=TRUE;
     Self.SaveAsOption.Enabled:=FALSE;
     Self.CloseCurrentOption.Enabled:=TRUE;
   end
else
if Self.ActiveMDIChild is TWeaponsPriorityFixer then
   begin
     Self.SaveOption.Enabled:=TRUE;
     Self.SaveAsOption.Enabled:=FALSE;
     Self.CloseCurrentOption.Enabled:=TRUE;
   end
else
{add any new options just above here}
   begin
     Self.SaveOption.Enabled:=FALSE;
     Self.SaveAsOption.Enabled:=FALSE;
     Self.CloseCurrentOption.Enabled:=FALSE;
   end;{elseif}
end;{if not ShuttingDown}
End;


procedure TMainMenuForm.CloseCurrentOptionClick(Sender: TObject);
begin
if Self.ActiveMDIChild is TForm then       {returns FALSE if it's NIL}
   TForm(Self.ActiveMDIChild).Close;
end;

procedure TMainMenuForm.SaveAsOptionClick(Sender: TObject);
begin
if Self.ActiveMDIChild is TChildBotlibForm then
  TChildBotlibForm(ActiveMDIChild).SaveAsBtnClick(Sender);
end;

END.
